import tkinter
try:
	import urllib.request
	def fetch_file(url,name):
		print("fetching",name,"...")
		with open(name,"wb") as target:
			request = urllib.request.Request(url=url,headers={"User-Agent": "Mozilla/5.0"})
			file = urllib.request.urlopen(request)
			target.write(file.read())
		print("done!")
except:
	print("urllib would not load. Cannot fetch remote files")
	def fetch_file(url,name):
		raise Exception("urllib unloaded. Cannot fetch files")
import random
from time import sleep
import math
from math import sin, cos, radians
def show_dialogue(text):
	root = tkinter.Tk()
	root.title("info")
	info_label = tkinter.Label(root,text=text)
	close_button = tkinter.Button(root,text="close",command=lambda : root.quit())
	info_label.pack()
	close_button.pack()
	root.mainloop()
def rotate_point(x,y,angle):
	angle = radians(angle)
	new_x = (x*cos(angle)) - (y*sin(angle))
	new_y = (x*sin(angle)) + (y*cos(angle))
	return new_x, new_y
class RenderPolygon():
	def __init__(self,offsets,canvas,**kwargs):
		self.offsets = offsets # offset from centre
		self.canvas = canvas
		self.centre = [10,10]
		self.angle = 0
		self.angle_degrees = 0
		self.recalculate()
		#create_lines
		self.lines = []
		for i, point in enumerate(self.points):
			prev_point = self.points[i-1] #wraps around to connect first and last point which is a handy coincidence
			#unpack points into args of function
			line = self.canvas.create_line(*prev_point,*point)
			self.lines.append(line)
	def recalculate(self):
		self.points = []
		for point in self.offsets:
			x = point[0]
			y = point[1]
			new_x = (x*cos(self.angle)) - (y*sin(self.angle))
			new_y = (x*sin(self.angle)) + (y*cos(self.angle))
			absolute_point = [
				new_x+self.centre[0],
				new_y+self.centre[1]
			]
			self.points.append(absolute_point)
	def render(self):
		for i, point in enumerate(self.points):
			prev_point = self.points[i-1]
			line = self.lines[i]
			self.canvas.coords(line,*prev_point,*point)
	def moveto(self,x,y):
		self.centre = [x,y]
		self.recalculate()
		self.render()
	def move(self,x,y):
		self.centre = [self.centre[0]+x,self.centre[1]+y]
		self.recalculate()
		self.render()
	def set_angle(self,angle):
		self.angle_degrees = angle
		self.angle = radians(angle)
		self.recalculate()
		self.render()
	def get_angle(self):
		return self.angle_degrees
	def print(self):
		for p in self.points:
			print(p)


class RenderText():
	def __init__(self,canvas,text):
		self.coords = [0,0]
		self.offsets = [0,0]
		self.canvas = canvas
		self.text_object = self.canvas.create_text(*self.coords,text=text)
		self.angle = 0
	def set_angle(self,angle):
		self.angle = angle
	def moveto(self,x,y):
		self.coords = [x,y]
	def recalculate(self):
		self.canvas.itemconfig(self.text_object,angle=self.angle)
		self.canvas.moveto(self.text_object,*self.coords)
		x1, y1, x2, y2 = self.canvas.bbox(self.text_object)
		centre_x = (x1+x2)/2
		centre_y = (y1+y2)/2
		self.offset = [centre_x-x1,centre_y-y1]
	def render(self):
		self.canvas.moveto(self.text_object,
			self.coords[0]-self.offset[0],
			self.coords[1]-self.offset[1]
		)
class Gui():
	def wheel_generate(self,radius,vertex_count):
		points = []
		unit = 360/vertex_count
		for i in range(vertex_count):
			angle = unit*i
			x,y = rotate_point(0,radius,angle)
			points.append([x,y])
		return points
	def __init__(self,names):
		self.x = 560
		self.y = 160
		self.radius = 170
		self.spoke_count = len(names)
		self.root = tkinter.Tk()
		self.root.geometry("800x600")
		self.root.title("wheel of fortune")
		self.canvas = tkinter.Canvas(self.root)
		self.create_text(names)
		self.canvas.pack(fill="both",expand=True)
		point_image = tkinter.PhotoImage(file="point.png")
		self.canvas.create_image(300,200,image=point_image)
		self.rim = RenderPolygon(self.wheel_generate(self.radius,self.spoke_count),self.canvas)
		spoke = self.wheel_generate(self.radius,self.spoke_count)
		origin_list = [[0,0] for i in range(self.spoke_count)]
		spoke_complete = spoke + origin_list
		spoke_complete[::2] = spoke
		spoke_complete[1::2] = origin_list
		#print(spoke_complete)
		self.spokes = RenderPolygon(spoke_complete,self.canvas)
		self.rim.moveto(self.x,self.y)
		self.spokes.moveto(self.x,self.y)
		self.mainloop()
	def mainloop(self):
		angle = 0
		angle = 0
		extra_time = 0.000001
		while True:
			sleep(extra_time)
			if extra_time >= 0.5:
				break
			extra_time *= 1.1
			angle += 10
			angle %= 360
			self.rim.set_angle(angle)
			self.spokes.set_angle(angle)
			self.update_text(angle)
			self.root.update()
		self.root.mainloop()
	def create_text(self,names):
		self.text_objects = []
		for name in names:
			self.text_objects.append(RenderText(self.canvas,name))
	def update_text(self,angle):
		unit = 360/len(self.text_objects)
		for i, name in enumerate(self.text_objects):
			x, y = rotate_point(0,self.radius/2,angle+(unit*i)+unit/2)
			x += self.x
			y += self.y
			name.set_angle(270-((angle+(unit*i))+(unit/2)))
			name.moveto(x,y)
			name.recalculate()
			name.render()
try:
	open("point.png","r")
except FileNotFoundError:
	fetch_file("https://stevens-server.co.uk/files/point.png/","point.png")
try:
	with open("names.txt","r") as names_doc:
		names = names_doc.read().strip("\n").split("\n")
except FileNotFoundError:
	with open("names.txt","w") as names_doc:
		names_doc.write("name 1\nname 2\nname 3\nname4\nname 5")
	#show_dialogue("You need to fill in the names.txt file")
	names = [f"name {i}" for i in range(5)]
random.shuffle(names)
Gui(names)
show_dialogue("credits:\nDavid Vincent Klaus Salamone - inspiration\nRose - The pointing image edit\nHarry - making the \"funcitoning\" prototype")
